@@ -11,7 +11,7 @@ from torch.autograd import Function
 from typing import List
 
 from ocnn.octree import Octree
-from ocnn.utils import scatter_add, xavier_uniform_
+from ocnn.utils import scatter_add, xavier_uniform_, resize_with_last_val, list2str
 from .octree2col import octree2col, col2octree
 from .octree_pad import octree_pad, octree_depad
 
@@ -24,8 +24,8 @@ class OctreeConvBase:
     super().__init__()
     self.in_channels = in_channels
     self.out_channels = out_channels
-    self.kernel_size = self.resize_with_last_val(kernel_size)
-    self.kernel = self.list2str(self.kernel_size)
+    self.kernel_size = resize_with_last_val(kernel_size)
+    self.kernel = list2str(self.kernel_size)
     self.stride = stride
     self.nempty = nempty
     self.max_buffer = max_buffer  # about 200M
@@ -35,24 +35,6 @@ class OctreeConvBase:
     self.out_conv = out_channels if self.is_conv_layer() else in_channels
     self.weights_shape = (self.kdim, self.in_conv, self.out_conv)
 
-  def resize_with_last_val(self, list_in, num=3):
-    r''' Resizes the number of elements of :attr:`list_in` to :attr:`num` with
-    the last element of :attr:`list_in` if its number of elements is smaller
-    than :attr:`num`.
-    '''
-
-    assert (type(list_in) is list and len(list_in) < num + 1)
-    for i in range(len(list_in), num):
-      list_in.append(list_in[-1])
-    return list_in
-
-  def list2str(self, list_in):
-    r''' Returns a string representation of :attr:`list_in`
-    '''
-
-    out = [str(x) for x in list_in]
-    return ''.join(out)
-
   def is_conv_layer(self):
     r''' Returns :obj:`True` to indicate this is a convolution layer.
     '''
@@ -116,7 +98,7 @@ class OctreeConvBase:
 
     # Check the shape of input data
     check = tuple(data.shape) == self.in_shape
-    torch._assert(check, 'The shape of input data is wrong.')
+    assert check, 'The shape of input data is wrong.'
 
     # Init the output data
     out = data.new_zeros(self.out_shape)
@@ -178,7 +160,8 @@ class OctreeConvBase:
 
     return out
 
-  def weight_gemm(self, out: torch.Tensor, data: torch.Tensor, grad: torch.Tensor):
+  def weight_gemm(
+          self, out: torch.Tensor, data: torch.Tensor, grad: torch.Tensor):
     r''' Computes the gradient of the weight matrix.
     '''
 
@@ -325,6 +308,15 @@ class OctreeConv(OctreeConvBase, torch.nn.Module):
     use_bias (bool): If True, add a bias term to the convolution.
     max_buffer (int): The maximum number of elements in the buffer, used when
         :attr:`direct_method` is False.
+
+  .. note::
+    Each non-empty octree node has exactly 8 children nodes, among which some
+    children nodes are non-empty and some are empty. If :attr:`nempty` is true,
+    the convolution is performed on non-empty octree nodes only, which is exactly
+    the same as SparseConvNet and MinkowsiNet; if :attr:`nempty` is false, the
+    convolution is performed on all octree nodes, which is essential for shape
+    reconstruction tasks and can also be used in classification and segmentation
+    (with slightly better performance and larger memory cost).
   '''
 
   def __init__(self, in_channels: int, out_channels: int,
