@@ -9,7 +9,7 @@ import torch
 import torch.nn.functional as F
 from typing import Union, List
 
-from ocnn.utils import meshgrid, scatter_add, cumsum
+from ocnn.utils import meshgrid, scatter_add, cumsum, trunc_div
 from .points import Points
 from .shuffled_key import xyz2key, key2xyz
 
@@ -76,7 +76,7 @@ class Octree:
     center_grid = self.rng_grid(2, 3)    # (8, 3)
     displacement = self.rng_grid(-1, 1)  # (27, 3)
     neigh_grid = center_grid.unsqueeze(1) + displacement  # (8, 27, 3)
-    parent_grid = torch.div(neigh_grid, 2, rounding_mode='trunc')
+    parent_grid = trunc_div(neigh_grid, 2)
     child_grid = neigh_grid % 2
     self.lut_parent = torch.sum(
         parent_grid * torch.tensor([9, 3, 1], device=device), dim=2)
@@ -231,7 +231,7 @@ class Octree:
     '''
 
     # check
-    torch._assert(depth <= self.full_depth, 'error')
+    assert depth <= self.full_depth, 'error'
 
     # node number
     num = 1 << (3 * depth)
@@ -463,11 +463,14 @@ class Octree:
     points = self.points[d] * scale - 1.0
     return Points(points, self.normals[d], self.features[d])
 
-  def to(self, device: Union[torch.device, str]):
+  def to(self, device: Union[torch.device, str], non_blocking: bool = False):
     r''' Moves the octree to a specified device.
 
     Args:
       device (torch.device or str): The destination device.
+      non_blocking (bool): If True and the source is in pinned memory, the copy
+          will be asynchronous with respect to the host. Otherwise, the argument
+          has no effect. Default: False.
     '''
 
     if isinstance(device, str):
@@ -478,7 +481,8 @@ class Octree:
       return self
 
     def list_to_device(prop):
-      return [p.to(device) if isinstance(p, torch.Tensor) else None for p in prop]
+      return [p.to(device, non_blocking=non_blocking)
+              if isinstance(p, torch.Tensor) else None for p in prop]
 
     # Construct a new Octree on the specified device
     octree = Octree(self.depth, self.full_depth, self.batch_size, device)
@@ -494,10 +498,10 @@ class Octree:
     octree.batch_nnum_nempty = self.batch_nnum_nempty.clone()
     return octree
 
-  def cuda(self):
+  def cuda(self, non_blocking: bool = False):
     r''' Moves the octree to the GPU. '''
 
-    return self.to('cuda')
+    return self.to('cuda', non_blocking)
 
   def cpu(self):
     r''' Moves the octree to the CPU. '''
@@ -528,7 +532,7 @@ def merge_octrees(octrees: List['Octree']):
     condition = (octrees[i].depth == octree.depth and
                  octrees[i].full_depth == octree.full_depth and
                  octrees[i].device == octree.device)
-    torch._assert(condition, 'The check of merge_octrees failed')
+    assert condition, 'The check of merge_octrees failed'
 
   # node num
   batch_nnum = torch.stack(
