@@ -38,11 +38,15 @@ class Points:
     self.normals = normals
     self.features = features
     self.labels = labels
-    self.device = points.device
     self.batch_id = batch_id
     self.batch_size = batch_size
+    self.device = points.device
     self.batch_npt = None   # valid after `merge_points`
 
+  @property
+  def npt(self):
+    return self.points.shape[0]
+
   def orient_normal(self, axis: str = 'x'):
     r''' Orients the point normals along a given axis.
 
@@ -73,7 +77,7 @@ class Points:
     non_zero = (factor != 0).all()
     all_ones = (factor == 1.0).all()
     non_uniform = (factor != factor[0]).any()
-    torch._assert(non_zero, 'The scale factor must not constain 0.')
+    assert non_zero, 'The scale factor must not constain 0.'
     if all_ones: return
 
     factor = factor.to(self.device)
@@ -121,10 +125,11 @@ class Points:
     '''
 
     axis_map = {'x': 0, 'y': 1, 'z': 2}
-    idx = axis_map[axis]
-    self.points[:, idx] *= -1.0
-    if self.normals is not None:
-      self.normals[:, idx] *= -1.0
+    for x in axis:
+      idx = axis_map[x]
+      self.points[:, idx] *= -1.0
+      if self.normals is not None:
+        self.normals[:, idx] *= -1.0
 
   def clip(self, min: float = -1.0, max: float = 1.0, esp: float = 0.01):
     r''' Clips the point cloud to :obj:`[min+esp, max-esp]` and returns the mask.
@@ -135,20 +140,38 @@ class Points:
       esp (float): The margin.
     '''
 
-    mask_min = torch.all(self.points > min + esp, dim=1)
-    mask_max = torch.all(self.points < max - esp, dim=1)
-    mask = torch.logical_and(mask_min, mask_max)
+    mask = self.inbox_mask(min + esp, max - esp)
+    tmp = self.__getitem__(mask)
+    self.__dict__.update(tmp.__dict__)
+    return mask
 
-    self.points = self.points[mask]
+  def __getitem__(self, mask: torch.Tensor):
+    r''' Slices the point cloud according a given :attr:`mask`.
+    '''
+
+    dummy_pts = torch.zeros(1, 3, device=self.device)
+    out = Points(dummy_pts, batch_size=self.batch_size)
+
+    out.points = self.points[mask]
     if self.normals is not None:
-      self.normals = self.normals[mask]
+      out.normals = self.normals[mask]
     if self.features is not None:
-      self.features = self.features[mask]
+      out.features = self.features[mask]
     if self.labels is not None:
-      self.labels = self.labels[mask]
+      out.labels = self.labels[mask]
     if self.batch_id is not None:
-      self.batch_id = self.batch_id[mask]
-      self.batch_npt = None  # TODO: Update batch_npt
+      out.batch_id = self.batch_id[mask]
+    return out
+
+  def inbox_mask(self, bbmin: Union[float, torch.Tensor] = -1.0,
+                 bbmax: Union[float, torch.Tensor] = 1.0):
+    r''' Returns a mask indicating whether the points are within the specified
+    bounding box or not.
+    '''
+
+    mask_min = torch.all(self.points > bbmin, dim=1)
+    mask_max = torch.all(self.points < bbmax, dim=1)
+    mask = torch.logical_and(mask_min, mask_max)
     return mask
 
   def bbox(self):
@@ -160,7 +183,8 @@ class Points:
     bbmax = self.points.max(dim=0)
     return bbmin[0], bbmax[0]
 
-  def normalize(self, bbmin: torch.Tensor, bbmax: torch.Tensor, scale: float = 1.0):
+  def normalize(self, bbmin: torch.Tensor, bbmax: torch.Tensor,
+                scale: float = 1.0):
     r''' Normalizes the point cloud to :obj:`[-scale, scale]`.
 
     Args:
@@ -173,11 +197,14 @@ class Points:
     box_size = (bbmax - bbmin).max() + 1.0e-6
     self.points = (self.points - center) * (2.0 * scale / box_size)
 
-  def to(self, device: Union[torch.device, str]):
+  def to(self, device: Union[torch.device, str], non_blocking: bool = False):
     r''' Moves the Points to a specified device.
 
     Args:
       device (torch.device or str): The destination device.
+      non_blocking (bool): If True and the source is in pinned memory, the copy
+          will be asynchronous with respect to the host. Otherwise, the argument
+          has no effect. Default: False.
     '''
 
     if isinstance(device, str):
@@ -190,71 +217,67 @@ class Points:
     # Construct a new Points on the specified device
     points = Points(torch.zeros(1, 3, device=device))
     points.batch_npt = self.batch_npt
-    points.points = self.points.to(device)
+    points.points = self.points.to(device, non_blocking=non_blocking)
     if self.normals is not None:
-      points.normals = self.normals.to(device)
+      points.normals = self.normals.to(device, non_blocking=non_blocking)
     if self.features is not None:
-      points.features = self.features.to(device)
+      points.features = self.features.to(device, non_blocking=non_blocking)
     if self.labels is not None:
-      points.labels = self.labels.to(device)
+      points.labels = self.labels.to(device, non_blocking=non_blocking)
     if self.batch_id is not None:
-      points.batch_id = self.batch_id.to(device)
+      points.batch_id = self.batch_id.to(device, non_blocking=non_blocking)
     return points
 
-  def cuda(self):
+  def cuda(self, non_blocking: bool = False):
     r''' Moves the Points to the GPU. '''
 
-    return self.to('cuda')
+    return self.to('cuda', non_blocking)
 
   def cpu(self):
     r''' Moves the Points to the CPU. '''
 
     return self.to('cpu')
 
-  def save(self, filename: str, save_batch: bool = False):
+  def save(self, filename: str, info: str = 'PNFL'):
     r''' Save the Points into npz or xyz files.
 
     Args:
       filename (str): The output filename.
-      save_batch (bool): Whether to save the batch index.
+      info (str): The infomation for saving: 'P' -> 'points', 'N' -> 'normals', 
+          'F' -> 'features', 'L' -> 'labels', 'B' -> 'batch_id'.
     '''
 
-    name = ['points']
-    out = [self.points.cpu().numpy()]
-    if self.normals is not None:
-      name.append('normals')
-      out.append(self.normals.cpu().numpy())
-    if self.features is not None:
-      name.append('features')
-      out.append(self.features.cpu().numpy())
-    if self.labels is not None:
-      name.append('labels')
-      labels = self.labels
-      if labels.dim() == 1:
-        labels = labels.unsqueeze(1)
-      out.append(labels.cpu().numpy())
-    if self.batch_id is not None and save_batch:
-      name.append('batch_id')
-      batch_id = self.batch_id
-      if batch_id.dim() == 1:
-        batch_id = batch_id.unsqueeze(1)
-      out.append(batch_id.cpu().numpy())
+    mapping = {
+        'P': ('points', self.points), 'N': ('normals', self.normals),
+        'F': ('features', self.features), 'L': ('labels', self.labels),
+        'B': ('batch_id', self.batch_id), }
+
+    names, outs = [], []
+    for key in info.upper():
+      name, out = mapping[key]
+      if out is not None:
+        names.append(name)
+        if out.dim() == 1:
+          out = out.unsqueeze(1)
+        outs.append(out.cpu().numpy())
 
     if filename.endswith('npz'):
-      out_dict = dict(zip(name, out))
+      out_dict = dict(zip(names, outs))
       np.savez(filename, **out_dict)
     elif filename.endswith('xyz'):
-      out_array = np.concatenate(out, axis=1)
+      out_array = np.concatenate(outs, axis=1)
       np.savetxt(filename, out_array, fmt='%.6f')
     else:
       raise ValueError
 
 
-def merge_points(points: List['Points']):
+def merge_points(points: List['Points'], update_batch_info: bool = True):
   r''' Merges a list of points into one batch.
 
   Args:
-    points (List[Octree]): A list of points to merge.
+    points (List[Octree]): A list of points to merge. The batch size of each
+        points in the list is assumed to be 1, and the :obj:`batch_size`, 
+        :obj:`batch_id`, and :obj:`batch_npt` in the points are ignored.
   '''
 
   out = Points(torch.zeros(1, 3))
@@ -266,8 +289,10 @@ def merge_points(points: List['Points']):
   if points[0].labels is not None:
     out.labels = torch.cat([p.labels for p in points], dim=0)
   out.device = points[0].device
-  out.batch_size = len(points)
-  out.batch_npt = torch.Tensor([p.points.shape[0] for p in points])
-  out.batch_id = torch.cat([p.points.new_full((p.points.shape[0], 1), i)
-                            for i, p in enumerate(points)], dim=0)
+
+  if update_batch_info:
+    out.batch_size = len(points)
+    out.batch_npt = torch.Tensor([p.npt for p in points]).long()
+    out.batch_id = torch.cat([p.points.new_full((p.npt, 1), i)
+                              for i, p in enumerate(points)], dim=0)
   return out
